#ifndef _CUBE_H
#define _CUBE_H

#include <stdint.h>
#include "common.h"
#include "uip/uip.h"



/*
		Hardwarespezifische Defines
		===========================      
SINGLE_LED
----------
Ist dieses Define gesetzt, wird jede LED einzeln adressiert und deren
Schaltzustand in der Interruptroutine an Decoder/Matrix ausgegeben.
Vorteil: die Helligkeit der LEDs untereinander ist unabhaengig von der
Konstellation der Schaltzustaende.
Nachteil: die Performance des Controllers geht runter, da oeffters die
ISR aufgerufen werden muss.
Ist das Define nicht gesetzt, werden die LEDs zeilenweise (zu je 8 LEDs)
angesteuert.
Nachteil: die Helligkeit der LEDs der gleichen Zeile variiert zu den LEDs
anderer Zeilen
*/
/*
#define SINGLE_LED
*/

/*
WITH_SCL
--------
Es hat sich als Vorteil erwiesen, kurz vor der Ausgabe der neuen LED-Zustaende
in der ISR, die vorhergehenden definiert zu loeschen, also Nullen in das 
Schieberegister des Decoders zu schieben und auszugeben. Dafuer gibt es 
2 Moeglichkeiten:
-den SCL-Eingang des Schieberesisters zu benutzen (Define gesetzt); dieser
 muss also verdrahtet sein
-softwareseitig Nullen ins Register schieben und SCL auf Hight verdrahten,
 was mal mein Ausgangszustand der Schaltung war (Define nicht gesetzt)
*/
#define WITH_SCL


/* TCP/IP-Port */
#define CUBE_PORT 12345

/* Cube-Dimension */
#define CUBE_X 8
#define CUBE_Y 8
#define CUBE_Z 8
#define CUBE_XYZ (CUBE_X * CUBE_Y * CUBE_Z)

/* Timer1-Werte berechnen */
#define CUBE_REFRESH    60						/* Bildwiederholfrequenz in Hz */
#define T1_PRESCALE     1024						/* Vorteiler des Timers        */
#define T1_MAX_COUNTER  65535						/* Maximalwert des Timers      */

#ifdef SINGLE_LED
#define T1_FREQ         (CUBE_REFRESH * CUBE_Y * CUBE_Z * CUBE_X)	/* Zeilenfrequenz              */
#else
#define T1_FREQ         (CUBE_REFRESH * CUBE_Y * CUBE_Z)		/* Zeilenfrequenz              */
#endif

#define T1_PRELOAD      (T1_MAX_COUNTER-(F_CPU/T1_PRESCALE/T1_FREQ))	/* resultierender Preloadwert  */


/* Befehlswoerter */
#define CMD_SET_LED       "SET_LED"
#define CMD_GET_XYZ       "GET_XYZ"
//#define CMD_GET_LED	  "GET_LED"
#define CMD_VERSION       "VERSION"
#define CMD_SET_PRELOAD   "SET_PRELOAD"
#define CMD_HELP          "HELP"


/* Zeitpunkt der Uebersetzung (wird im Makefile gesetzt) */
#ifndef BUILD_DATE
#define BUILD_DATE "unknown\n"
#endif
static const char build_date[] = BUILD_DATE;

/* Antwort auf Befehl GET_XYZ */
#define TOSTR(val) STR(val)
#define STR(val) #val
#define OUT_SET_XYZ "set_xyz " TOSTR(CUBE_X) " " TOSTR(CUBE_Y) " " TOSTR(CUBE_Z) "\n"
static const char out_set_xyz[] = OUT_SET_XYZ;

/* Antwort auf Befehle SET_LED */
#define OUT_UPDATE_OK "update_ok\n"
static const char out_update_ok[] = OUT_UPDATE_OK;

/* Prototypen */
void SplitCmdArg(char * line, char ** cmd, char ** args);
void cube_init(void);
void cube_main(void);

/* Typ-Definitionen */
struct cube_cmd_state {
	enum {CMD_SET_LED_RECEIVE, NEW_CMD} state;
	uint16_t  arg_pos;		/* naechste zu uebernehmende Position bei set_led */
	char     *textptr;
	uint16_t  textlen;
};


#endif
